/*

Name: XCheckJAXP

Version: 1.0
 
Description: an implementation of XCheck-Java with Java API for XML Processing (JAXP).

Author: Copyright (C) 2007 Massimo Franceschet (massimo.franceschet@dimi.uniud.it) - Dipartimento di Matematica e Informatica, Universita' degli Studi di Udine, Via delle Scienze, 206 - 33100 Udine (Italy)

License: GNU General Public License (GPL)    
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

import java.io.*;
import org.w3c.dom.*;
import org.w3c.dom.ls.*;
import javax.xml.*;
import javax.xml.parsers.*;
import javax.xml.xpath.*;
import org.xml.sax.SAXException;


/*************************************************************
Implement XCheck-Java with Java API for XML Processing (JAXP).
*************************************************************/
public class XCheckJAXP extends XCheckVoid {

  /***********************
  Parse the input document 
  ************************/
  public Document parseInputDocument(int j, String target) {
      
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder parser = null;
    try {
      parser = factory.newDocumentBuilder();
    } catch (ParserConfigurationException pce) {pce.printStackTrace();}
    parser.setErrorHandler(new LooseErrorHandler());
    File documentFile = new File(target);
    Document document = null;
    try {
      document = parser.parse(documentFile);
    } catch (IOException ioe) {
      printErrorMessage("There was a problem while parsing document: "
          + target, ioe.toString());
      errorMessageDocument[j] = ioe.toString();
      errorDocument[j] = true;
    } catch (SAXException se) {
      errorMessageDocument[j] = se.toString();
      errorDocument[j] = true;
    }
    return document;
  }

        
  /**************
  Execute a query 
  ***************/
  public Object executeInputQuery(int j, Document document, String query)  {
    
    XPathExpression queryExpression = null;
    XPath xpath = XPathFactory.newInstance().newXPath();

    /***************************
    Compile the query expression
    ****************************/
    try {
      queryExpression = xpath.compile(query);
        } catch (XPathExpressionException xee) {
          printErrorMessage("There was a problem while parsing query: " + 
              query, xee.toString()); 
          errorMessageQuery[j] = xee.toString(); 
          errorQuery[j] = true;
          return null;
        }
    
    /**********************************************
    Evaluate the query according to its result type
    ***********************************************/
    Object result = null;    
    if (queryType[j].equals("nodeset")) {
      try {
        result = (NodeList) 
            queryExpression.evaluate(document, XPathConstants.NODESET);
      } catch (XPathExpressionException xee) {xee.printStackTrace();}
    }
    
    if (queryType[j].equals("number")) {
      try {
        result = (Double) queryExpression.evaluate(document, XPathConstants.NUMBER);
      } catch (XPathExpressionException xee) {xee.printStackTrace();}
    }
    
    if (queryType[j].equals("string")) {
      try {
        result = (String) queryExpression.evaluate(document, XPathConstants.STRING);
      } catch (XPathExpressionException xee) {xee.printStackTrace();}
    }
    
    if (queryType[j].equals("boolean")) {
      try {
        result = (Boolean) queryExpression.evaluate(document, XPathConstants.BOOLEAN);
      } catch (XPathExpressionException xee) {xee.printStackTrace();}
    }
    return result;
  }
  
  
  /*************************
  Serialize the query result 
  **************************/
  public void serializeResult(int j, Object result, String resultFileName, 
                              Document document) {
    
    DOMImplementationLS implementation = 
        (DOMImplementationLS) document.getImplementation();
    LSOutput output = implementation.createLSOutput();
    LSSerializer serializer = implementation.createLSSerializer();
    //serializer.getDomConfig().setParameter("format-pretty-print",true);
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder parser = null;
    try {
      parser = factory.newDocumentBuilder();
    } catch (ParserConfigurationException pce) {pce.printStackTrace();}
    Document resultDocument = parser.newDocument();
    Element resultNode = resultDocument.createElement("answer");
    if (queryType[j].equals("nodeset")) {
      NodeList resultNodeSet = (NodeList) result;
      int numResult = resultNodeSet.getLength();
      for(int k = 0; k < numResult; k++) {
        resultNode.appendChild(
            resultDocument.importNode(resultNodeSet.item(k), true));
      }
    }
    if (queryType[j].equals("number")) {
      Double resultNumber = (Double) result;
      resultNode.appendChild(
          resultDocument.createTextNode(resultNumber.toString()));
    }
    if (queryType[j].equals("string")) {
      String resultString = (String) result;
      resultNode.appendChild(
          resultDocument.createTextNode(resultString));
    }
    if (queryType[j].equals("boolean")) {
      Boolean resultBoolean = (Boolean) result;
      resultNode.appendChild(
          resultDocument.createTextNode(resultBoolean.toString()));
    }
    resultDocument.appendChild(resultNode);
    FileWriter outputStream = null;
    try {
      outputStream = new FileWriter(resultFileName);
    } catch (IOException ioe) {ioe.printStackTrace();}
    output.setCharacterStream(outputStream);
    serializer.write(resultDocument, output);
    try {
      outputStream.close();
    } catch (IOException ioe) {ioe.printStackTrace();}
  }
  
  
  
  /*********************************************************************
  Run XCheck-Java with Java API for XML Processing (JAXP) by calling the
  method check inherited from class XCheckVoid
  *********************************************************************/
  public static void main(String[] args) {
    
    XCheckJAXP jaxp = new XCheckJAXP();
    jaxp.check(args);
    
  }


}

